/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState } from 'react';
import { Node, Handle, Position, NodeProps } from 'reactflow';
import styles from './entity.module.scss';
import { SchemaReactflowNodeWithFunctions } from '../../../model/reactflow';
import { QueryElementTypes } from '@graphpolaris/shared/lib/querybuilder';
import { SchemaEntityPopup } from './SchemaEntityPopup';
import { Popup } from '@graphpolaris/shared/lib/components/Popup';

/**
 * EntityNode is the node that represents the database entities.
 * It has four handles on the top, bottom, right and left to connect edges.
 * It also has a text to display the entity name and show the amount of attributes it contains.
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const EntityNode = React.memo(({ id, selected, data }: NodeProps<SchemaReactflowNodeWithFunctions>) => {
  const [openPopup, setOpenPopup] = useState(false);

  /**
   * adds drag functionality in order to be able to drag the entityNode to the schema
   * @param event React Mouse drag event
   */
  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    event.dataTransfer.setData('application/reactflow', JSON.stringify({ type: QueryElementTypes.Entity, name: id }));
    event.dataTransfer.effectAllowed = 'move';
  };

  /**
   * displays the NodeQualityPopup when clicked, and hides them when clicked again
   */
  const onClickToggleNodeQualityPopup = (): void => {
    data.toggleNodeQualityPopup(id);
  };

  /**
   * displays the attribute analytics popup menu when clicked, and hides them when clicked again
   */
  const onClickToggleAttributeAnalyticsPopupMenu = (): void => {
    data.toggleAttributeAnalyticsPopupMenu(id);
  };

  return (
    <>
      {openPopup && (
        <Popup open={openPopup} hAnchor="left" className="-top-10" offset="-9rem">
          <SchemaEntityPopup data={data} onClose={() => setOpenPopup(false)} />
        </Popup>
      )}
      <div
        className={`border-l-2 border-l-entity-600 min-w-[8rem] text-[0.8rem] ${selected ? 'bg-slate-400' : 'bg-offwhite-200'}`}
        onDragStart={(event) => onDragStart(event)}
        onDragStartCapture={(event) => onDragStart(event)}
        onMouseDownCapture={(event) => {
          if (!event.shiftKey) event.stopPropagation();
        }}
        onClickCapture={(event) => {
          setOpenPopup(!openPopup);
        }}
        draggable
      >
        {/* <div
        className={styles.entityNodeAttributesBox}
        onClick={() => onClickToggleAttributeAnalyticsPopupMenu()}
        style={{
          borderBottomColor: theme.palette.custom.elements.entityBase[0],
          width: calcWidthEntityNodeBox(data.attributes.length, data.nodeCount) + 'ch',
          background: `linear-gradient(-90deg, 
           ${theme.palette.custom.nodesBase} 0%, 
           ${theme.palette.custom.nodesBase} ${calculateAttributeQuality(data)}%, 
           ${theme.palette.custom.elements.entitySecond[0]} ${calculateAttributeQuality(data)}%)`,
        }}
      >
        <span
          style={{
            paddingLeft: '5px',
            float: 'left',
          }}
        >
          A
        </span>
        <span className={styles.nodeSpan}>{data.attributes.length}</span>
      </div> */}
        {/* <div
        className={styles.entityNodeNodesBox}
        onClick={() => onClickToggleNodeQualityPopup()}
        style={{
          borderBottomColor: theme.palette.custom.elements.entityBase[0],
          width: calcWidthEntityNodeBox(data.attributes.length, data.nodeCount) + 'ch',
          background: `linear-gradient(-90deg, 
           ${theme.palette.custom.nodesBase} 0%, 
           ${theme.palette.custom.nodesBase} ${calculateEntityQuality(data)}%, 
           ${theme.palette.custom.elements.entitySecond[0]} ${calculateEntityQuality(data)}%)`,
        }}
      >
        <span
          style={{
            paddingLeft: '5px',
            float: 'left',
          }}
        >
          N
        </span>
        <span className={styles.nodeSpan}>{data.nodeCount}</span>
      </div> */}
        {/* <Handle
        style={{ pointerEvents: 'none' }}
        id="entitySourceLeft"
        position={Position.Left}
        className={styles.handleTriangleLeft}
        type="source"
        // hidden={Array.from(data.handles).includes('entitySourceLeft') ? false : true}
      ></Handle> */}
        <Handle
          style={{ pointerEvents: 'none' }}
          id="entityTargetLeft"
          position={Position.Left}
          className={styles.handleTriangleLeft}
          type="target"
          // hidden={Array.from(data.handles).includes('entityTargetLeft') ? false : true}
        ></Handle>
        <Handle
          style={{ pointerEvents: 'none' }}
          id="entitySourceRight"
          position={Position.Right}
          className={styles.handleTriangleRight}
          type="source"
          // hidden={Array.from(data.handles).includes('entitySourceRight') ? false : true}
        ></Handle>
        {/* <Handle
        style={{ pointerEvents: 'none' }}
        id="entityTargetRight"
        position={Position.Right}
        type="target"
        // hidden={Array.from(data.handles).includes('entityTargetRight') ? false : true}
      ></Handle> */}
        {/* <Handle
        style={{ pointerEvents: 'none' }}
        id="entitySourceTop"
        position={Position.Top}
        type="source"
        // hidden={Array.from(data.handles).includes('entitySourceTop') ? false : true}
      ></Handle>
      <Handle
        style={{ pointerEvents: 'none' }}
        id="entityTargetTop"
        position={Position.Top}
        type="target"
        // hidden={Array.from(data.handles).includes('entityTargetTop') ? false : true}
      ></Handle> */}
        {/* <Handle
        style={{ pointerEvents: 'none' }}
        id="entitySourceBottom"
        position={Position.Bottom}
        type="source"
        // hidden={Array.from(data.handles).includes('entitySourceBottom') ? false : true}
      ></Handle>
      <Handle
        style={{ pointerEvents: 'none' }}
        id="entityTargetBottom"
        position={Position.Bottom}
        type="target"
        // hidden={Array.from(data.handles).includes('entityTargetBottom') ? false : true}
      ></Handle> */}
        <div className="p-2 py-1">
          <span className="">{id}</span>
        </div>
      </div>
    </>
  );
});

EntityNode.displayName = 'EntityNode';
