import { facetNodes, updateEdges } from './attributeFaceting';
import GeoEdge from './edge';
import GeoNode from './node';

export default class GeoGraph {
  public nodeMap: Record<string, GeoNode>;
  public edgeMap: Record<string, GeoEdge>;
  public cache = {};

  constructor() {
    this.nodeMap = {};
    this.edgeMap = {};

    this.cache = {
      nodes: {},
      edges: {},
    };
  }

  addNode(node: GeoNode) {
    this.nodeMap[node.id] = node;
  }

  findNode(nodeID: string | number) {
    return this.nodeMap[nodeID];
  }

  getNodes = () => Object.values(this.nodeMap);

  batchAddNodes(nodes: GeoNode[]) {
    nodes.forEach((node) => this.addNode(node));
    this.updateCache();
  }

  addEdge(edge: GeoEdge) {
    const sourceNode: GeoNode = this.findNode(edge.getSourceNodeId());
    const targetNode: GeoNode = this.findNode(edge.getTargetNodeId());
    if (!sourceNode || !targetNode) return;
    const key = `${sourceNode.id}-${targetNode.id}`;
    if (!this.edgeMap[key]) {
      edge.source = sourceNode.getCoordinates();
      edge.target = targetNode.getCoordinates();
      this.edgeMap[key] = edge;
    } else {
      this.edgeMap[key].count++;
    }
  }

  getEdges = () => Object.values(this.edgeMap);

  batchAddEdges(edges: GeoEdge[]) {
    edges.forEach((edge) => this.addEdge(edge));
    this.updateCache();
  }

  groupBy(dimension: string) {
    const nodes = facetNodes(Object.values(this.nodeMap), dimension);
    const edges = updateEdges(nodes, Object.values(this.edgeMap));
    return {
      nodes: nodes,
      edges: edges,
    };
  }

  updateCache() {
    this.cache = {
      nodes: this.nodeMap,
      edges: this.edgeMap,
    };
  }

  calculateCenterPoint() {
    const nodes: GeoNode[] = Object.values(this.nodeMap);
    const totalNodes: number = nodes.length;
    let centerLon: number = 0;
    let centerLat: number = 0;

    for (const node of nodes) {
      const coordinates: number[] = node.getCoordinates();
      centerLon += coordinates[0];
      centerLat += coordinates[1];
    }

    return {
      longitude: centerLon / totalNodes,
      latitude: centerLat / totalNodes,
    };
  }
}
