import { GraphType, Node, Edge, Coordinate } from './Types';

export default class GraphModel implements GraphType {
  nodeMap: { [id: string]: Node };
  edgeMap: { [id: string]: Edge };
  graphInfo: {
    nodeAttributes: { [attribute: string]: any };
    edgeAttributes: { [attribute: string]: any };
  };

  constructor() {
    this.nodeMap = {};
    this.edgeMap = {};

    this.graphInfo = {
      nodeAttributes: {},
      edgeAttributes: {},
    };
  }

  getNode(id: string): Node | null {
    return this.nodeMap[id] ?? null;
  }

  getNodes(): Node[] {
    return Object.values(this.nodeMap);
  }

  getNodeLocation(id: string): Coordinate | null {
    const node: Node | null = this.getNode(id);
    return node ? [node.attributes.long, node.attributes.lat] : null;
  }

  getConnectedEdges(nodeId: string) {
    const node = this.getNode(nodeId);
    if (node) {
      const edgeIds = node.connectedEdges;
      if (edgeIds.length > 0) {
        const edges = edgeIds.map((id: string) => this.edgeMap[id]);
        return edges;
      } else {
        return [];
      }
    } else {
      return [];
    }
  }

  getEdge(id: string): Edge | null {
    return this.edgeMap[id] ?? null;
  }

  getEdges(): Edge[] {
    return Object.values(this.edgeMap);
  }

  consumeMessageFromBackend(queryResult: { [key: string]: any }) {
    queryResult.nodes.map((node: Node) => {
      const connectedEdges: string[] = [];
      queryResult.edges.map((edge: Edge) => {
        this.edgeMap[edge.id] = edge;
        this.collectAttributeInfo(this.graphInfo.edgeAttributes, edge);
        if (edge.from === node.id || edge.to === node.id) {
          connectedEdges.push(edge.id);
        }
      });

      this.nodeMap[node.id] = { ...node, connectedEdges };
      this.collectAttributeInfo(this.graphInfo.nodeAttributes, node);
    });
  }

  getGraphInfo(): {
    nodeAttributes: { [attribute: string]: any };
    edgeAttributes: { [attribute: string]: any };
  } {
    return this.graphInfo;
  }

  collectAttributeInfo(attributeCollection: { [key: string]: any }, entity: any) {
    for (const attribute in entity) {
      const value = entity[attribute];
      const dataType = typeof value;

      if (dataType == 'object') {
        this.collectAttributeInfo(attributeCollection, value);
      } else {
        if (attributeCollection[attribute]) {
          attributeCollection[attribute].count++;
          if (dataType == 'number') {
            const currentRange = attributeCollection[attribute].values;
            attributeCollection[attribute].values = [Math.min(currentRange[0], value), Math.max(currentRange[1], value)];
          } else if (dataType == 'string') {
            attributeCollection[attribute].values.add(value);
          }
        } else {
          if (dataType == 'number') {
            attributeCollection[attribute] = {
              count: 1,
              dataType,
              values: [value, value],
            };
          } else if (dataType == 'string') {
            attributeCollection[attribute] = {
              count: 1,
              dataType,
              values: new Set([value]),
            };
          } else if (dataType == 'boolean') {
            attributeCollection[attribute] = {
              count: 1,
              dataType,
            };
          }
        }
      }
    }
  }
}
